; Copyright (c) 2021, rrc-open-firmware FM10K-Documentation Contributors
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are met:
;
;     * Redistributions of source code must retain the above copyright notice,
;       this list of conditions and the following disclaimer.
;     * Redistributions in binary form must reproduce the above copyright
;       notice, this list of conditions and the following disclaimer in the
;       documentation and/or other materials provided with the distribution.
;     * Neither the name of the copyright holder nor the names of its contributors
;       may be used to endorse or promote products derived from this software
;       without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
; SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
; OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

interrupt_handler:
    MOV api_BSM_STATUS, 0x000f0011
    WRITE rrcc_STACK_POINTER, rrcc_STACK_START     ; Reset Stack pointer to its starting position
    WRITE R0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ; Zero R0 - RF
    WRITE api_RE_RESET_MASK_STATUS_1, 0x00000000, 0x00000000 ; 64-bit write

    .takeSoftResetLock: CALL @lock_soft_reset_take
    BNE RRET, 1, 0x1, @.takeSoftResetLock ; Failed to take lock, keep retrying in busy loop

    CALL @interrupt_handle_each_source
    ; TODO this

    END


lock_soft_reset_take:
    PNE api_SOFT_RESET_LOCK_STATE, 0, 0x3, 20000, 50, @.failTakeLock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3
        BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.retryLock
            MOV RRET, 1 ; Success
            RET
        .retryLock:
        PNE api_SOFT_RESET_LOCK_STATE, 0, 0x3, 20000, 50, @.failTakeLock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3
        BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.failTakeLock
            MOV RRET, 1 ; Success
            RET
    .failTakeLock:
    MOV RRET, 0 ; Failure
    SET api_SW_LOCK_ERR_STATUS, 0x00000100, 0x00000100
    BEQ api_SW_LOCK_ERR_STATUS, 0xfe00, 0xfe00, @.returnFail
        ADD api_SW_LOCK_ERR_STATUS, api_SW_LOCK_ERR_STATUS, 0x200
        RET
    .returnFail: SET api_SW_LOCK_ERR_STATUS, 0x0000, 0xfe00
    ADD api_SW_LOCK_ERR_STATUS, api_SW_LOCK_ERR_STATUS, 0x200
    RET

lock_soft_reset_release:
    BNE api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_NVM, 0x3, @.return  ; If we hold the lock
        SET api_SOFT_RESET_LOCK_STATE, platform_SOFT_RESET_LOCK_FREE, 0x3       ; Release lock
    .return:
    RET

interrupt_handle_each_source:
    BNE GLOBAL_INTERRUPT_DETECT_0, 0x00000001, 0x00000001, @.skipPep0 ; Need to handle PEP 0
        MOV api_BSM_STATUS, 0x00000011
        CALL @interrupt_set_clocks_once_PEP0
    .skipPep0:
    RET


.constant tmp_PEP_CLK_MASK 0x00700000
.constant tmp_PEP_STAT_MASK 0x00000100
.constant tmp_PEP_RESET_MASK 0x00000010
.constant tmp_PEP_ACTIVE_MASK 0x00002000
interrupt_set_clocks_once_PEP0:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP0_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_RESET_MASK 0x00000020
.constant tmp_PEP_ACTIVE_MASK 0x00004000
interrupt_set_clocks_once_PEP1:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP1_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_CLK_MASK 0x03800000
.constant tmp_PEP_STAT_MASK 0x00000200
.constant tmp_PEP_RESET_MASK 0x00000040
.constant tmp_PEP_ACTIVE_MASK 0x00008000
interrupt_set_clocks_once_PEP2:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP2_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_RESET_MASK 0x00000080
.constant tmp_PEP_ACTIVE_MASK 0x00010000
interrupt_set_clocks_once_PEP3:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP3_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_CLK_MASK 0x1c000000
.constant tmp_PEP_STAT_MASK 0x00000400
.constant tmp_PEP_RESET_MASK 0x00000100
.constant tmp_PEP_ACTIVE_MASK 0x00020000
interrupt_set_clocks_once_PEP4:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP4_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_RESET_MASK 0x00000200
.constant tmp_PEP_ACTIVE_MASK 0x00040000
interrupt_set_clocks_once_PEP5:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP5_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_CLK_MASK 0xe0000000
.constant tmp_PEP_STAT_MASK 0x00000800
.constant tmp_PEP_RESET_MASK 0x00000400
.constant tmp_PEP_ACTIVE_MASK 0x00080000
interrupt_set_clocks_once_PEP6:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP6_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET

.constant tmp_PEP_RESET_MASK 0x00000800
.constant tmp_PEP_ACTIVE_MASK 0x00100000
interrupt_set_clocks_once_PEP7:
    BEQ CHIP_VERSION, 0x00, 0x7f, @.skipReset
        BEQ PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK, @.skipReset ; If Mode for PEP group = PCIE_REFCLK
            BNE PCIE_CLK_STAT, 0x00000000, tmp_PEP_STAT_MASK, @.skipReset ; If RefclkSel != 0
                AND platform_PEP7_PCIE_CLK_CTRL, PCIE_CLK_CTRL, tmp_PEP_CLK_MASK
                SET SOFT_RESET, tmp_PEP_RESET_MASK, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 1
                SET PCIE_CLK_CTRL, 0x00000000, tmp_PEP_CLK_MASK; Set Mode for PEP group = PCIE_REFCLK
                SET SOFT_RESET, 0x00000000, tmp_PEP_RESET_MASK ; Set PCIeReset for PEP = 0
    .skipReset: SET SOFT_RESET, tmp_PEP_ACTIVE_MASK, tmp_PEP_ACTIVE_MASK ; Set PCIeActive for PEP = 1
    RET
